# decompyle3 version 3.3.2
# Python bytecode 3.7 (3394)
# Decompiled from: Python 3.8.1 (default, Jan  3 2020, 22:44:00) 
# [GCC 8.3.0]
# Embedded file name: ulang\__init__.py
# Size of source mod 2**32: 82682 bytes
from ulang.parser.core import Parser
from ulang.runtime.main import main
__version__ = '0.2.2'
__all__ = [
 'Parser',
 'main']